# -*- coding: utf-8 -*-

# @module: PyMaOrb.py
# @version: 1.3
# @summary: This module communicates with Matrix Orbital display(s).

# @author: Morten André Steinsland
# @contact: morten.a.steinsland@gmail.com
# @note: http://steinslandsolutions.no/pymaorb/

# @copyright: Original Copyright 2010 Morten André Steinsland.
# @copyright: Revisions copyright 2015 John Peedle john@peedle.org.uk
# @license: Creative Commons Attribution-Noncommercial-Share Alike 3.0 Norway License. http://creativecommons.org/licenses/by-nc-sa/3.0/

# DISCLAIMER:
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
# THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

# @note: Only been tested on LK204-25-USB model, and also some functions may be missing,
#        broken, not done, undocumented or not tested properly.

# An example of a character map dictionary to be put in LCDDisplay.CHARMAP
# note: Far from all the characters and symbols the display can show are
# represented here, this is just an example. Consult the display manual
# for a full character listing.
CHARS = {'é':   '\x82',
         'É':   '\x90',
         'æ':   '\x91',
         'Æ':   '\x92',
         'ø':   '\xAF',
         'Ø':   '\xAE',
         'å':   '\x86',
         'Å':   '\x8F',
         '±':   '\x10',
         '°':   '\xB2',
         '£':   '\xA5',
         '†':   '\xD1',
         '™':   '\xD0',
         '¥':   '\xA6',
         '∑':   '\xDA',
         'arrowup':      '\xC5',    # This creates a thin arrow
         'arrowdown':    '\xC6',    #
         'arrowright':   '\xC7',    #
         'arrowleft':    '\xC8',    #
         'ARROWDOWN':    '\xF5',    # This creates a fat arrow
         'ARROWRIGHT':   '\xF6',    #
         'ARROWLEFT':    '\xF7'}    #


import serial

class LCDDisplay:    
    def __init__(self,comport,baudrate=19200,cols=20,rows=4):
        self.CHARMAP = CHARS            # Set this to a character map dictionary
        self.MAXCOLS = cols             # Number of columns on the display
        self.MAXROWS = rows             # Number of rows on the display
        
        self.DISPLAY = serial.Serial(comport,baudrate)
        self.SetRemember(False)         # Set to False while tinkering
        
    def Write(self,data):
        """Sends commands/text to the display."""
        if isinstance(data, int):
            self.DISPLAY.write(chr(data))
        else:
            self.DISPLAY.write(self.CodeIt(data))
    
    ##[ TEXT ]###################################################################
    def ClearScreen(self):
        """Clears the screen."""
        self.Write(chr(254) + chr(88))    
        
    def AutoScrollOn(self,enabled=True):
        """When auto scrolling is on, it causes the display to shift the entire
        display’s contents up to make room for a new line of text when the text
        reaches the end of the last row."""
        if enabled == True:
            self.Write(chr(254) + chr(81))
        if enabled == False:
            self.Write(chr(254) + chr(82))
    
    def AutoLineWrapOn(self,enabled=True):
        if enabled == True:
            self.Write(chr(254 ) + chr(67))
        if enabled == False:
            self.Write(chr(254) + chr(68))
            
    def SetCursorPos(self,column=1,row=1):
        self.Write(chr(254) + chr(71) + chr(column) + chr(row))
        
    def GoHome(self):
        self.Write(chr(254) + chr(72))
        
    def MoveCursorBack(self):
        self.Write(chr(254) + chr(76))
        
    def MoveCursorForward(self):
        self.Write(chr(254) + chr(77))
        
    def UnderlineCursorOn(self,enabled=False):
        if enabled == True:
            self.Write(chr(254) + chr(74))
        if enabled == False:
            self.Write(chr(254) + chr(75))
            
    def BlinkingBlockCursorOn(self,enabled=False):
        if enabled == True:
            self.Write(chr(254) + chr(83))
        if enabled == False:
            self.Write(chr(254) + chr(84))
    
    ## Change Startup Screen
    # @bug: "s6" (or other strange chars) can appear if the last characters are not space
    def ChangeStartupScreen(self,text):
        """Changes the startup screen.
        Remembered: Yes"""
        self.Write(chr(254) + chr(64) + text)
    
    def CodeIt(self,string):
        """This will convert the characters found in self.CHARMAP to its corresponding (hex)code"""
        newstring = string
        dictkeys = self.CHARMAP.keys()
        for character in dictkeys:
            newstring = newstring.replace(character,self.CHARMAP[character])
        
        return newstring
    
    
    ##[ CUSTOM CHARACTERS ]#####################################################    
    def CreateCustomCharacter(self, refID, data):
        """Create a custom character.
        Remembered: No"""
        if (len(data) != 8) or not (isinstance(data, tuple)):
            raise ValueError('Data must be a tuple of 8 binaries!')
        else:
            self.Write(chr(254) + chr(78) + chr(refID))
            for eachline in data:
                self.Write(chr(int(eachline, 2)))
    
    def SaveCustomCharacters(self, bank, refID, data):
        if (len(data) != 8) or not (isinstance(data, tuple)):
            raise ValueError('Data must be a tuple of 8 binaries!')
        else:
            self.Write(chr(254) + chr(193) + chr(bank) + chr(refID))
            for eachline in data:
                self.Write(chr(int(eachline, 2)))
    
    def LoadCustomCharacters(self, bank):
        """Load custom characters into the volatile memory so that they may be used."""
        self.Write(chr(254) + chr(192) + chr(bank))
        
    
    def SaveStartupScreenCustomCharacters(self, refID, data):
        if (len(data) != 8) or not (isinstance(data, tuple)):
            raise ValueError('Data must be a tuple of 8 binaries!')
        else:
            self.Write(chr(254) + chr(194) + chr(refID))
            for eachline in data:
                self.Write(chr(int(eachline, 2)))

    
    ##[ SPECIAL CHARACTERS ]#####################################################
    def InitializeMediumDigits(self):
        self.Write(chr(254) + chr(109))
        
    def PlaceMediumDigit(self, digit, row=1, col=1):
        self.Write(chr(254) + chr(111) + chr(row) + chr(col) + chr(digit))
    
    def InitializeLargeDigits(self):
        self.Write(chr(254) + chr(110))
        
    def PlaceLargeDigit(self, digit, col=1):
        self.Write(chr(254) + chr(35) + chr(col) + chr(digit))

    def InitializeHorizontalBar(self):
        self.Write(chr(254) + chr(104))
        
    def PlaceHorizontalBarGraph(self, col=1, row=1, direction=0, length=1):
        self.Write(chr(254) + chr(124) + chr(col) + chr(row) + chr(direction) + chr(length))
        
    def InitializeNarrowVerticalBar(self):
        self.Write(chr(254) + chr(115))
        
    def InitializeWideVerticalBar(self):
        self.Write(chr(254) + chr(118))
        
    def PlaceVerticalBarGraph(self, col=1, length=1):
        self.Write(chr(254) + chr(61) + chr(col) + chr(length))

    
    ##[ GENERAL PURPOSE OUTPUT ]##################################################
    def GeneralPurposeOutputOn(self, enabled, gponumber):
        if enabled == True:
            self.Write(chr(254) + chr(87) + chr(gponumber))
        if enabled == False:
            self.Write(chr(254) + chr(86) + chr(gponumber))
            
    def SetStartupGPOState(self, gponumber, state):
        if (state == 1) or (state == 0):
            self.Write(chr(254) + chr(195) + chr(gponumber) + chr(state))


    
    ##[ DALLAS 1-WIRE ]###########################################################
    def SearchForDallas1WireDevice(self):
        pass
    
    def Dallas1WireTransaction(self):
        pass
    
    ##[ KEYPAD ]##################################################################    
    def AutoTransmitKeyPressesOn(self,enabled=True):
        if enabled == True:
            self.Write(chr(254) + chr(65))
        if enabled == False:
            self.Write(chr(254) + chr(79))
            
    def PollKeyPress(self):
        """This command returns any buffered key presses via the serial interface.
        The host system must be set up to receive key codes. When the display
        receives this command, it will immediately return any buffered key
        presses which may have not been read already. If there is more than one
        key press buffered, then the high order bit (MSB) of the returned key
        code will be set (1). If this is the only buffered key press, then the MSB
        will be cleared (0). If there are no buffered key presses, then the
        returned code will be 0x00. Please note that to make use of this
        command, the “Auto Transmit Key Presses” mode should be off.
        
        Remembered: No """
        self.Write(chr(254) + chr(38))
        
    def ClearKeyBuffer(self):
        """This command clears any unread key presses. In a menu application, if
        the user presses a key which changes the menu context, any following
        key presses may be inaccurate and can be cleared out of the buffer
        between menu changes to prevent jumping around the menu tree. It may
        also be used, in effect, to reset the keypad in case the host application
        resets for whatever reason.
        
        Remembered: No"""
        self.Write(chr(254) + chr(69))
        
    def SetDebounceTime(self,time=8):
        """This command sets the time between key press and key read. All key
        types with the exception of latched piezo switches will ’bounce’ for a
        varying time, depending on their physical characteristics. The [time]
        value is in increments of 6.554ms. The default debounce time for the
        module is 8 (about 52ms), which is adequate for most membrane
        keypads. 'time' = Debounce time in increments of 6.554ms (0 - 255)
        
        Remembered: Yes
        Default: 8 """
        self.Write(chr(254) + chr(85) +chr(time))
        
    def SetAutoRepeatMode(self,mode):
        self.Write(chr(254) + chr(126) +chr(mode))
    
    def AutoRepeatModeOff(self):
        """This command turns auto repeat mode off. See Set Auto Repeat Mode.
        
        Remembered: No """
        self.Write(chr(254) + chr(96))
    
    def AssignKeypadCode(self,kdown,kup):
        pass
    
    ##[ DISPLAY ]#################################################################
    def Backlight(self,enabled=True,waitminutes=0):
        "Turns the backlight On or Off"
        if enabled == True:
            self.Write(chr(254) + chr(66) +chr(waitminutes))
        if enabled == False:
            self.Write(chr(254) + chr(70) +chr(waitminutes))
            
    def BacklightColour(self, red, green, blue):
	self.Write(chr(254) + chr(130) + chr(red) + chr(green) + chr(blue))

    def SetBrightness(self,brightness=255):
        """This command sets the display [brightness]. If the remember function is
        on, this command acts the same as ’Set and Save Brightness’."""
        self.Write(chr(254) + chr(153) +chr(brightness))
    
    def SetAndSaveBrightness(self,brightness=255):
        self.Write(chr(254) + chr(152) +chr(brightness))
        
    def SetContrast(self,contrast=255):
        """Contrast value (0 to 255)."""
        self.Write(chr(254) + chr(80) +chr(contrast))
        
    def SetAndSaveContrast(self,contrast=255):
        self.Write(chr(254) + chr(145) +chr(contrast))
        
    def SetKeypadLEDOff():
	self.Write(chr(254) + chr(155))

    def SetKeypadBrightness(self, brightness):
	self.Write(chr(254) + chr(156) + chr(brightness))

    ##[ DATA SECURITY ]###########################################################
    def SetRemember(self,enabled=False):
        """This command allows you to switch the remember function on and off.
        To use the remember function, set remember to on, then set all of the
        settings that you wish to save, settings that are listed as ’Remember:
        Yes’ support being saved into the non-volatile memory. After you have
        set all of the commands that you wish to save, you may then cycle the
        power and check the display settings to ensure that all the settings have
        been saved. If you wish to use remember again after cycling the power,
        you must set it to on again.
        
        False: Do not remember, True: Remember"""
        if enabled == True:
            self.Write(chr(254) + chr(147) +chr(1))
        if enabled == False:
            self.Write(chr(254) + chr(147) +chr(0))
            
    def DataLock(self):
        pass
    
    def SetAndSaveDataLock(self):
        pass
    
    def WriteCustomerData(self):
        pass
    
    def ReadCustomerData(self):
        pass
        
    ##[ MISCELLANEOUS ]###########################################################
    def ReadVersionNumber(self):
        self.Write(chr(254) + chr(54))
        version = self.DISPLAY.read(1).encode("hex")
        return version
    
    def ReadModuleType(self):
        self.Write(chr(254) + chr(55))
        modtype = self.DISPLAY.read(1).encode("hex")
        return modtype

